#ifndef LIST_H_INCLUDED
#define LIST_H_INCLUDED

/* a doubly-linked circular linked list  */ 
/* mostly used as a browsable FIFO queue */ 

struct list_node
{
  struct list_node *pPrev, *pNext;
  char *sText;
};

struct list
{
  struct list_node *pHead;
  struct list_node *pBrowse;
  unsigned nEntries, nMaxEntries;
};

#ifdef __cplusplus
extern "C"
}
#endif

struct list *List_Create(unsigned nMaxEntries); /* O(1) time */ 
struct list *List_Destroy(struct list *pList); /* O(N) time */ 
struct list_node *List_FindNode(struct list *pList, const char *sText); /* O(N*M) time */ 
struct list_node *List_FindPrefixNode(struct list *pList, const char *sPrefix, unsigned cch); /* O(N*M) time */ 
void List_QueueInsert(struct list *pList, const char *sText); /* O(1) time */ 
void List_QueueInsertUnique(struct list *pList, const char *sText); /* O(N*M) time */ 

/* where N = nEntries, M = strlen */ 

#ifdef __cplusplus
}
#endif

#endif /* LIST_H_INCLUDED */
